from django.contrib.auth import load_backend, BACKEND_SESSION_KEY
from django.contrib.auth.models import Group
from django.contrib.auth.backends import RemoteUserBackend
from django.contrib.auth.middleware import RemoteUserMiddleware

class RemoteUserAttrMiddleware(RemoteUserMiddleware):
    group_prefix = 'ext:'


    def update_user_groups(self, request):
        user = request.user

        ext_group_count = request.META.get("REMOTE_USER_GROUP_N", None)
        if ext_group_count is None:
            return

        current_groups = {}
        for g in user.groups.filter(name__startswith=self.group_prefix):
            current_groups[g.name] = g

        for i in range(1, int(ext_group_count) + 1):
            if request.META.get("REMOTE_USER_GROUP_" + str(i), None):
                g = self.group_prefix + request.META["REMOTE_USER_GROUP_" + str(i)]
                if current_groups.has_key(g):
                    del current_groups[g]
                else:
                    g_obj = Group.objects.filter(name=g)
                    if g_obj:
                        user.groups.add(g_obj[0])

        for g in current_groups.values():
            user.groups.remove(g.id)

    def process_request(self, request):
        if hasattr(request, 'user') and request.user.is_authenticated() and \
          request.META.get(self.header, None) and \
          request.user.get_username() == self.clean_username(request.META[self.header], request):
            stored_backend = load_backend(request.session.get(BACKEND_SESSION_KEY, ''))
            if isinstance(stored_backend, RemoteUserBackend):
                user = request.user
                need_save = False
                email = request.META.get("REMOTE_USER_EMAIL", None)
                if email is not None:
                    user.email = email
                    need_save = True
                firstname = request.META.get("REMOTE_USER_FIRSTNAME", None)
                if firstname is not None:
                    user.first_name = firstname
                    need_save = True
                lastname = request.META.get("REMOTE_USER_LASTNAME", None)
                if lastname is not None:
                    user.last_name = lastname
                    need_save = True
                if need_save or request.META.get("REMOTE_USER_GROUP_N", None):
                    self.update_user_groups(request)
                    user.save()
